@@ -20,7 +20,6 @@ package org.apache.spark.network.crypto;
 import java.nio.ByteBuffer;
 import java.nio.channels.WritableByteChannel;
 import java.security.GeneralSecurityException;
-import java.util.Arrays;
 import java.util.Map;
 import java.util.Random;
 
@@ -64,10 +63,9 @@ public class AuthEngineSuite {
 
   @Test
   public void testAuthEngine() throws Exception {
-    AuthEngine client = new AuthEngine("appId", "secret", conf);
-    AuthEngine server = new AuthEngine("appId", "secret", conf);
 
-    try {
+    try (AuthEngine client = new AuthEngine("appId", "secret", conf);
+         AuthEngine server = new AuthEngine("appId", "secret", conf)) {
       AuthMessage clientChallenge = client.challenge();
       AuthMessage serverResponse = server.response(clientChallenge);
       client.deriveSessionCipher(clientChallenge, serverResponse);
@@ -75,12 +73,9 @@ public class AuthEngineSuite {
       TransportCipher serverCipher = server.sessionCipher();
       TransportCipher clientCipher = client.sessionCipher();
 
-      assertTrue(Arrays.equals(serverCipher.getInputIv(), clientCipher.getOutputIv()));
-      assertTrue(Arrays.equals(serverCipher.getOutputIv(), clientCipher.getInputIv()));
+      assertArrayEquals(serverCipher.getInputIv(), clientCipher.getOutputIv());
+      assertArrayEquals(serverCipher.getOutputIv(), clientCipher.getInputIv());
       assertEquals(serverCipher.getKey(), clientCipher.getKey());
-    } finally {
-      client.close();
-      server.close();
     }
   }
 
@@ -208,9 +203,8 @@ public class AuthEngineSuite {
 
   @Test
   public void testEncryptedMessage() throws Exception {
-    AuthEngine client = new AuthEngine("appId", "secret", conf);
-    AuthEngine server = new AuthEngine("appId", "secret", conf);
-    try {
+    try (AuthEngine client = new AuthEngine("appId", "secret", conf);
+         AuthEngine server = new AuthEngine("appId", "secret", conf)) {
       AuthMessage clientChallenge = client.challenge();
       AuthMessage serverResponse = server.response(clientChallenge);
       client.deriveSessionCipher(clientChallenge, serverResponse);
@@ -228,17 +222,13 @@ public class AuthEngineSuite {
         emsg.transferTo(channel, emsg.transferred());
       }
       assertEquals(data.length, channel.length());
-    } finally {
-      client.close();
-      server.close();
     }
   }
 
   @Test
   public void testEncryptedMessageWhenTransferringZeroBytes() throws Exception {
-    AuthEngine client = new AuthEngine("appId", "secret", conf);
-    AuthEngine server = new AuthEngine("appId", "secret", conf);
-    try {
+    try (AuthEngine client = new AuthEngine("appId", "secret", conf);
+         AuthEngine server = new AuthEngine("appId", "secret", conf)) {
       AuthMessage clientChallenge = client.challenge();
       AuthMessage serverResponse = server.response(clientChallenge);
       client.deriveSessionCipher(clientChallenge, serverResponse);
@@ -274,9 +264,6 @@ public class AuthEngineSuite {
       assertEquals(testDataLength, emsg.transferTo(channel, emsg.transferred()));
       assertEquals(emsg.transferred(), emsg.count());
       assertEquals(4, channel.length());
-    } finally {
-      client.close();
-      server.close();
     }
   }
 }
