@@ -28,6 +28,7 @@ import scala.util.control.NonFatal
 
 import org.apache.spark._
 import org.apache.spark.TaskState.TaskState
+import org.apache.spark.errors.SparkCoreErrors
 import org.apache.spark.internal.{config, Logging}
 import org.apache.spark.internal.config._
 import org.apache.spark.resource.ResourceInformation
@@ -125,7 +126,7 @@ private[spark] class TaskSetManager(
   val weight = 1
   val minShare = 0
   var priority = taskSet.priority
-  var stageId = taskSet.stageId
+  val stageId = taskSet.stageId
   val name = "TaskSet_" + taskSet.id
   var parent: Pool = null
   private var totalResultSize = 0L
@@ -157,6 +158,14 @@ private[spark] class TaskSetManager(
   // same time for a barrier stage.
   private[scheduler] def isBarrier = taskSet.tasks.nonEmpty && taskSet.tasks(0).isBarrier
 
+  // Barrier tasks that are pending to launch in a single resourceOffers round. Tasks will only get
+  // launched when all tasks are added to this pending list in a single round. Otherwise, we'll
+  // revert everything we did during task scheduling.
+  private[scheduler] val barrierPendingLaunchTasks = new HashMap[Int, BarrierPendingLaunchTask]()
+
+  // Record the last log time of the barrier TaskSetManager that failed to get all tasks launched.
+  private[scheduler] var lastResourceOfferFailLogTime = clock.getTimeMillis()
+
   // Store tasks waiting to be scheduled by locality preferences
   private[scheduler] val pendingTasks = new PendingTasksByLocality()
 
@@ -222,9 +231,6 @@ private[spark] class TaskSetManager(
    */
   private[scheduler] var myLocalityLevels = computeValidLocalityLevels()
 
-  // Time to wait at each level
-  private[scheduler] var localityWaits = myLocalityLevels.map(getLocalityWait)
-
   // Delay scheduling variables: we keep track of our current locality level and the time we
   // last reset the locality wait timer, and move up a level when localityWaits[curLevel] expires.
   // We then move down if we manage to launch a "more local" task when resetting the timer
@@ -232,6 +238,9 @@ private[spark] class TaskSetManager(
   private var currentLocalityIndex = 0 // Index of our current locality level in validLocalityLevels
   private var lastLocalityWaitResetTime = clock.getTimeMillis()  // Time we last reset locality wait
 
+  // Time to wait at each level
+  private[scheduler] var localityWaits = myLocalityLevels.map(getLocalityWait)
+
   override def schedulableQueue: ConcurrentLinkedQueue[Schedulable] = null
 
   override def schedulingMode: SchedulingMode = SchedulingMode.NONE
@@ -302,7 +311,7 @@ private[spark] class TaskSetManager(
         // Speculatable task should only be launched when at most one copy of the
         // original task is running
         if (!successful(index)) {
-          if (copiesRunning(index) == 0) {
+          if (copiesRunning(index) == 0 && !barrierPendingLaunchTasks.contains(index)) {
             return Some(index)
           } else if (speculative && copiesRunning(index) == 1) {
             return Some(index)
@@ -410,17 +419,22 @@ private[spark] class TaskSetManager(
    * @param execId the executor Id of the offered resource
    * @param host  the host Id of the offered resource
    * @param maxLocality the maximum locality we want to schedule the tasks at
+   * @param taskCpus the number of CPUs for the task
+   * @param taskResourceAssignments the resource assignments for the task
    *
-   * @return Tuple containing:
-   *         (TaskDescription of launched task if any, rejected resource due to delay scheduling?)
+   * @return Triple containing:
+   *         (TaskDescription of launched task if any,
+   *         rejected resource due to delay scheduling?,
+   *         dequeued task index)
    */
   @throws[TaskNotSerializableException]
   def resourceOffer(
       execId: String,
       host: String,
       maxLocality: TaskLocality.TaskLocality,
+      taskCpus: Int = sched.CPUS_PER_TASK,
       taskResourceAssignments: Map[String, ResourceInformation] = Map.empty)
-    : (Option[TaskDescription], Boolean) =
+    : (Option[TaskDescription], Boolean, Int) =
   {
     val offerExcluded = taskSetExcludelistHelperOpt.exists { excludeList =>
       excludeList.isNodeExcludedForTaskSet(host) ||
@@ -439,77 +453,113 @@ private[spark] class TaskSetManager(
         }
       }
 
+      var dequeuedTaskIndex: Option[Int] = None
       val taskDescription =
         dequeueTask(execId, host, allowedLocality)
           .map { case (index, taskLocality, speculative) =>
-        // Found a task; do some bookkeeping and return a task description
-        val task = tasks(index)
-        val taskId = sched.newTaskId()
-        // Do various bookkeeping
-        copiesRunning(index) += 1
-        val attemptNum = taskAttempts(index).size
-        val info = new TaskInfo(taskId, index, attemptNum, curTime,
-          execId, host, taskLocality, speculative)
-        taskInfos(taskId) = info
-        taskAttempts(index) = info :: taskAttempts(index)
-        if (legacyLocalityWaitReset && maxLocality != TaskLocality.NO_PREF) {
-          resetDelayScheduleTimer(Some(taskLocality))
-        }
-        // Serialize and return the task
-        val serializedTask: ByteBuffer = try {
-          ser.serialize(task)
-        } catch {
-          // If the task cannot be serialized, then there's no point to re-attempt the task,
-          // as it will always fail. So just abort the whole task-set.
-          case NonFatal(e) =>
-            val msg = s"Failed to serialize task $taskId, not attempting to retry it."
-            logError(msg, e)
-            abort(s"$msg Exception during serialization: $e")
-            throw new TaskNotSerializableException(e)
-        }
-        if (serializedTask.limit() > TaskSetManager.TASK_SIZE_TO_WARN_KIB * 1024 &&
-          !emittedTaskSizeWarning) {
-          emittedTaskSizeWarning = true
-          logWarning(s"Stage ${task.stageId} contains a task of very large size " +
-            s"(${serializedTask.limit() / 1024} KiB). The maximum recommended task size is " +
-            s"${TaskSetManager.TASK_SIZE_TO_WARN_KIB} KiB.")
-        }
-        addRunningTask(taskId)
-
-        // We used to log the time it takes to serialize the task, but task size is already
-        // a good proxy to task serialization time.
-        // val timeTaken = clock.getTime() - startTime
-        val tName = taskName(taskId)
-        logInfo(s"Starting $tName ($host, executor ${info.executorId}, " +
-          s"partition ${task.partitionId}, $taskLocality, ${serializedTask.limit()} bytes) " +
-          s"taskResourceAssignments ${taskResourceAssignments}")
-
-        sched.dagScheduler.taskStarted(task, info)
-        new TaskDescription(
-          taskId,
-          attemptNum,
-          execId,
-          tName,
-          index,
-          task.partitionId,
-          addedFiles,
-          addedJars,
-          addedArchives,
-          task.localProperties,
-          taskResourceAssignments,
-          serializedTask)
-      }
+            dequeuedTaskIndex = Some(index)
+            if (legacyLocalityWaitReset && maxLocality != TaskLocality.NO_PREF) {
+              resetDelayScheduleTimer(Some(taskLocality))
+            }
+            if (isBarrier) {
+              barrierPendingLaunchTasks(index) =
+                BarrierPendingLaunchTask(
+                  execId,
+                  host,
+                  index,
+                  taskLocality,
+                  taskResourceAssignments)
+              // return null since the TaskDescription for the barrier task is not ready yet
+              null
+            } else {
+              prepareLaunchingTask(
+                execId,
+                host,
+                index,
+                taskLocality,
+                speculative,
+                taskCpus,
+                taskResourceAssignments,
+                curTime)
+            }
+          }
       val hasPendingTasks = pendingTasks.all.nonEmpty || pendingSpeculatableTasks.all.nonEmpty
       val hasScheduleDelayReject =
         taskDescription.isEmpty &&
           maxLocality == TaskLocality.ANY &&
           hasPendingTasks
-      (taskDescription, hasScheduleDelayReject)
+      (taskDescription, hasScheduleDelayReject, dequeuedTaskIndex.getOrElse(-1))
     } else {
-      (None, false)
+      (None, false, -1)
     }
   }
 
+  def prepareLaunchingTask(
+      execId: String,
+      host: String,
+      index: Int,
+      taskLocality: TaskLocality.Value,
+      speculative: Boolean,
+      taskCpus: Int,
+      taskResourceAssignments: Map[String, ResourceInformation],
+      launchTime: Long): TaskDescription = {
+    // Found a task; do some bookkeeping and return a task description
+    val task = tasks(index)
+    val taskId = sched.newTaskId()
+    // Do various bookkeeping
+    copiesRunning(index) += 1
+    val attemptNum = taskAttempts(index).size
+    val info = new TaskInfo(
+      taskId, index, attemptNum, task.partitionId, launchTime,
+      execId, host, taskLocality, speculative)
+    taskInfos(taskId) = info
+    taskAttempts(index) = info :: taskAttempts(index)
+    // Serialize and return the task
+    val serializedTask: ByteBuffer = try {
+      ser.serialize(task)
+    } catch {
+      // If the task cannot be serialized, then there's no point to re-attempt the task,
+      // as it will always fail. So just abort the whole task-set.
+      case NonFatal(e) =>
+        val msg = s"Failed to serialize task $taskId, not attempting to retry it."
+        logError(msg, e)
+        abort(s"$msg Exception during serialization: $e")
+        throw SparkCoreErrors.failToSerializeTaskError(e)
+    }
+    if (serializedTask.limit() > TaskSetManager.TASK_SIZE_TO_WARN_KIB * 1024 &&
+      !emittedTaskSizeWarning) {
+      emittedTaskSizeWarning = true
+      logWarning(s"Stage ${task.stageId} contains a task of very large size " +
+        s"(${serializedTask.limit() / 1024} KiB). The maximum recommended task size is " +
+        s"${TaskSetManager.TASK_SIZE_TO_WARN_KIB} KiB.")
+    }
+    addRunningTask(taskId)
+
+    // We used to log the time it takes to serialize the task, but task size is already
+    // a good proxy to task serialization time.
+    // val timeTaken = clock.getTime() - startTime
+    val tName = taskName(taskId)
+    logInfo(s"Starting $tName ($host, executor ${info.executorId}, " +
+      s"partition ${task.partitionId}, $taskLocality, ${serializedTask.limit()} bytes) " +
+      s"taskResourceAssignments ${taskResourceAssignments}")
+
+    sched.dagScheduler.taskStarted(task, info)
+    new TaskDescription(
+      taskId,
+      attemptNum,
+      execId,
+      tName,
+      index,
+      task.partitionId,
+      addedFiles,
+      addedJars,
+      addedArchives,
+      task.localProperties,
+      taskCpus,
+      taskResourceAssignments,
+      serializedTask)
+  }
+
   def taskName(tid: Long): String = {
     val info = taskInfos.get(tid)
     assert(info.isDefined, s"Can not find TaskInfo for task (TID $tid)")
@@ -724,6 +774,11 @@ private[spark] class TaskSetManager(
    */
   def handleSuccessfulTask(tid: Long, result: DirectTaskResult[_]): Unit = {
     val info = taskInfos(tid)
+    // SPARK-37300: when the task was already finished state, just ignore it,
+    // so that there won't cause successful and tasksSuccessful wrong result.
+    if(info.finished) {
+      return
+    }
     val index = info.index
     // Check if any other attempt succeeded before this and this attempt has not been handled
     if (successful(index) && killedByOtherAttempt.contains(tid)) {
@@ -766,6 +821,7 @@ private[spark] class TaskSetManager(
         s"on ${info.host} (executor ${info.executorId}) ($tasksSuccessful/$numTasks)")
       // Mark successful and stop if all the tasks have succeeded.
       successful(index) = true
+      numFailures(index) = 0
       if (tasksSuccessful == numTasks) {
         isZombie = true
       }
@@ -789,6 +845,7 @@ private[spark] class TaskSetManager(
       if (!successful(index)) {
         tasksSuccessful += 1
         successful(index) = true
+        numFailures(index) = 0
         if (tasksSuccessful == numTasks) {
           isZombie = true
         }
@@ -803,7 +860,9 @@ private[spark] class TaskSetManager(
    */
   def handleFailedTask(tid: Long, state: TaskState, reason: TaskFailedReason): Unit = {
     val info = taskInfos(tid)
-    if (info.failed || info.killed) {
+    // SPARK-37300: when the task was already finished state, just ignore it,
+    // so that there won't cause copiesRunning wrong result.
+    if (info.finished) {
       return
     }
     removeRunningTask(tid)
@@ -975,7 +1034,7 @@ private[spark] class TaskSetManager(
     // so we would need to rerun these tasks on other executors.
     if (isShuffleMapTasks && !env.blockManager.externalShuffleServiceEnabled && !isZombie) {
       for ((tid, info) <- taskInfos if info.executorId == execId) {
-        val index = taskInfos(tid).index
+        val index = info.index
         // We may have a running task whose partition has been marked as successful,
         // this partition has another task completed in another stage attempt.
         // We treat it as a running task and will call handleFailedTask later.
@@ -1035,7 +1094,7 @@ private[spark] class TaskSetManager(
    * by the TaskScheduler.
    *
    */
-  override def checkSpeculatableTasks(minTimeToSpeculation: Int): Boolean = {
+  override def checkSpeculatableTasks(minTimeToSpeculation: Long): Boolean = {
     // No need to speculate if the task set is zombie or is from a barrier stage. If there is only
     // one task we don't speculate since we don't have metrics to decide whether it's taking too
     // long or not, unless a task duration threshold is explicitly provided.
@@ -1089,6 +1148,8 @@ private[spark] class TaskSetManager(
   }
 
   private def getLocalityWait(level: TaskLocality.TaskLocality): Long = {
+    if (legacyLocalityWaitReset && isBarrier) return 0
+
     val localityWait = level match {
       case TaskLocality.PROCESS_LOCAL => config.LOCALITY_WAIT_PROCESS
       case TaskLocality.NODE_LOCAL => config.LOCALITY_WAIT_NODE
@@ -1162,6 +1223,9 @@ private[spark] object TaskSetManager {
   // The user will be warned if any stages contain a task that has a serialized size greater than
   // this.
   val TASK_SIZE_TO_WARN_KIB = 1000
+
+  // 1 minute
+  val BARRIER_LOGGING_INTERVAL = 60000
 }
 
 /**
@@ -1192,3 +1256,16 @@ private[scheduler] class PendingTasksByLocality {
   // Set containing all pending tasks (also used as a stack, as above).
   val all = new ArrayBuffer[Int]
 }
+
+private[scheduler] case class BarrierPendingLaunchTask(
+    execId: String,
+    host: String,
+    index: Int,
+    taskLocality: TaskLocality.TaskLocality,
+    assignedResources: Map[String, ResourceInformation]) {
+  // Stored the corresponding index of the WorkerOffer which is responsible to launch the task.
+  // Used to revert the assigned resources (e.g., cores, custome resources) when the barrier
+  // task set doesn't launch successfully in a single resourceOffers round.
+  var assignedOfferIndex: Int = _
+  var assignedCores: Int = 0
+}
